@@ -2,6 +2,7 @@
 #include <QDir>
 #include <QUrl>
 #include <QMap>
+#include <QtDebug>
 #include <QTemporaryFile>
 #include <QtNetwork/QNetworkAccessManager>
 #include <QtNetwork/QNetworkReply>
@@ -26,6 +27,23 @@ PlaylistManager::PlaylistManager(QWidget* parent)
     downloadmanager = new QNetworkAccessManager(this);
     connect(downloadmanager, SIGNAL(finished(QNetworkReply*)),
             this, SLOT(playlistDownloadFinished(QNetworkReply*)));
+    qDebug () << "Database drivers: " << QSqlDatabase::drivers();
+    cachedb = QSqlDatabase::addDatabase("QSQLITE");
+    cachedb.setDatabaseName(getCacheDbName());
+    cachedb.open();
+    if (cachedb.tables().indexOf("metadata") == -1)
+    {
+        QSqlQuery q (cachedb);
+       
+        q.exec("CREATE TABLE IF NOT EXISTS metadata(id INTEGER PRIMARY KEY,filename TEXT,artist TEXT,title TEXT,album TEXT)");
+        q.exec("CREATE INDEX fnameind ON metadata(filename)");
+    }
+}
+
+PlaylistManager::~PlaylistManager()
+{
+    cachedb.commit();
+    cachedb.close();
 }
 
 int PlaylistManager::indexOf(const Phonon::MediaSource &s) const
@@ -167,6 +185,7 @@ void PlaylistManager::metaStateChanged(Phonon::State newState, Phonon::State /*o
         if (!items[index].title.size())
             items[index].title = metaData.value("TITLE");
         items[index].album = metaData.value("ALBUM");
+        writeCache(items[index].uri, items[index].album, items[index].artist, items[index].title);
 /*        items[index].year = metaData.value("DATE");
         items[index].genre = metaData.value("GENRE");
         qDebug () << "Meta " << metaData;
@@ -182,6 +201,16 @@ void PlaylistManager::metaStateChanged(Phonon::State newState, Phonon::State /*o
     }*/
     while (index + 1 < items.size ())
     {
+        QStringList l = checkCache(items[index + 1].uri);
+        if (l.size() == 3)
+        {
+            items[index + 1].album = l[0];
+            items[index + 1].artist = l[1];
+            items[index + 1].title = l[2];
+            emit itemUpdated(index + 1);
+            index++;
+            continue;
+        }
         if (!items[index + 1].title.size() || !items[index + 1].album.size() || !items[index + 1].artist.size ())
         {
             metaInformationResolver->setCurrentSource(items[index + 1].source);
@@ -547,3 +576,44 @@ bool PlaylistManager::moveItemDown (int i)
     }
     return false;
 }
+
+QString PlaylistManager::getCacheDbName () const
+{
+    QString ret = QDir::homePath() + QDir::separator() + ".tomamp-cache";
+#ifdef Q_WS_MAEMO_5
+    ret = QDir::homePath() + QDir::separator() + "MyDocs/.tomamp-cache";
+#endif
+    return ret;
+}
+
+QStringList PlaylistManager::checkCache (const QString& fname)
+{
+    QSqlQuery q (cachedb);
+    q.prepare("SELECT artist,album,title FROM metadata WHERE filename=:fname");
+    q.bindValue(":fname", fname);
+    q.exec ();
+    if (!q.first ())
+        return QStringList();
+    QSqlRecord r = q.record();
+    if (r.isEmpty())
+        return QStringList();
+    else
+    {
+        QStringList ret;
+        ret << r.value("album").toString() << r.value("artist").toString() << r.value("title").toString();
+        qDebug () << "Returned from cache for " << fname << " => " << ret;
+        return ret;
+    }
+}
+
+void PlaylistManager::writeCache (const QString& fname, const QString& album, const QString& artist, const QString& title)
+{
+    QSqlQuery q (cachedb);
+    q.prepare("INSERT INTO metadata (filename,artist,album,title) VALUES (:fname,:art,:alb,:tit)");
+    q.bindValue(":fname", fname);
+    q.bindValue(":alb", album);
+    q.bindValue(":art", artist);
+    q.bindValue(":tit", title);
+    q.exec ();
+    cachedb.commit();
+}
